/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: SoundHandler.java,v 1.2 2005/08/19 03:18:11 ppoi Exp $
 */
package tsukuba_bunko.peko.scenario.stage;

import	org.xml.sax.Attributes;

import	tsukuba_bunko.peko.Logger;

import tsukuba_bunko.peko.canvas.stage.AudioPlayer;
import	tsukuba_bunko.peko.scenario.PSMLUtil;


/**
 * <samp>start-bgm</samp>, <samp>stop-bgm</samp>, <samp>start-se</samp>,
 * <samp>stop-se</samp> vf <code>ElementHandler</code> łB
 * @author	$Author: ppoi $
 * @version	$Revision: 1.2 $ $Date: 2005/08/19 03:18:11 $
 */
public class SoundHandler	extends StageElementHandler	{

	/**
	 * <code>SoundHandler</code> ̃CX^X쐬܂B
	 */
	public SoundHandler()
	{
		super();
	}


//
//	ContentHandler ̎
//
	public void startDocument()
	{
	}

	public void endDocument()
	{
	}

	public void startElement( String namespaceURI, String localName, String qName, Attributes attrs )
	{
		String	id = PSMLUtil.getAttributeValue( attrs, "id" );
		if( id == null )	{
			Logger.error( MessageIDs.SCN3003E, new Object[]{getSceneContext().getCurrentPath()} );
			return;
		}

		String	clipName = null;
		String	loop = null;
		boolean	lp = false;

		StageCoordinator	coordinator = getStageCoordinator();
		if( "play-bgm".equals(localName) )	{
			clipName = PSMLUtil.getAttributeValue( attrs, "clip" );
			if( clipName == null )	{
				Logger.warn( MessageIDs.SCN3004W, new Object[]{getSceneContext().getCurrentPath()} );
				return;
			}

			loop = PSMLUtil.getAttributeValue( attrs, "loop" );
			if( loop != null )	{
				lp = (loop.equalsIgnoreCase("true") || loop.equalsIgnoreCase("yes") || loop.equalsIgnoreCase("on"));
			}
			else	{
				lp = true;
			}

			coordinator.playBGM( id, clipName, lp );
		}
		else if( "play-se".equals(localName) )	{
			clipName = PSMLUtil.getAttributeValue( attrs, "clip" );
			if( clipName == null )	{
				Logger.warn( MessageIDs.SCN3004W, new Object[]{getSceneContext().getCurrentPath()} );
				return;
			}

			loop = PSMLUtil.getAttributeValue( attrs, "loop" );
			if( loop != null )	{
				lp = (loop.equalsIgnoreCase("true") || loop.equalsIgnoreCase("yes") || loop.equalsIgnoreCase("on"));
			}
			else	{
				lp = false;
			}

			coordinator.playSE( id, clipName, lp );
		}
		else if( "stop-bgm".equals(localName) )	{
			int	fadeoutMode = AudioPlayer.STOP_WITH_ASYNC_FADEOUT;
			String	fadeout = PSMLUtil.getAttributeValue( attrs, "fadeout" );
			if( fadeout != null )	{
				if( "sync".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_WITH_SYNC_FADEOUT;
				}
				else if( "none".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_IMMEDIATELY;
				}
				else if( "async".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_WITH_ASYNC_FADEOUT;
				}
				else	{
					Logger.warn( MessageIDs.SCN3007W, new Object[]{getSceneContext().getCurrentPath(), "async"} );
				}
			}
			coordinator.stopBGM( id, fadeoutMode );
		}
		else if( "stop-se".equals(localName) )	{
			int	fadeoutMode = AudioPlayer.STOP_WITH_ASYNC_FADEOUT;
			String	fadeout = PSMLUtil.getAttributeValue( attrs, "fadeout" );
			if( fadeout != null )	{
				if( "sync".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_WITH_SYNC_FADEOUT;
				}
				else if( "none".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_IMMEDIATELY;
				}
				else if( "async".equals(fadeout) )	{
					fadeoutMode = AudioPlayer.STOP_WITH_ASYNC_FADEOUT;
				}
				else	{
					Logger.warn( MessageIDs.SCN3007W, new Object[]{getSceneContext().getCurrentPath(), "async"} );
				}
			}
			coordinator.stopSE( id, fadeoutMode );
		}
	}
}
